# Tkabber presence spy module, version 1.0
# Copyright (C) 2003 Maciek Pasternacki <maciekp@japhy.fnord.org>
#
# Traces presence messages, logging them into tkabber window
# and/or file.
#
# Usage: # source file in ~/.tkabber/config.tcl (outside of postload):
#        source "~/.tkabber/spy.tcl"
#        # If you want to log presence messages to file, set filename:
#        set ::spy::filename "~/.tkabber/logs/presence.log"
#        # Now, either run tkabber from within tkcon and run
#        # ::spy::open_window from there to open spy window,
#        # or bind ::spy::open_window into menu or toolbar,
#        # for example like this:
#        proc menuload {desc} {
#            return [concat $desc \
#                        [list "&Plugins" {} {} 1 \
#                             [list \
#                                  [list command "Spy window" {} {} {} \
#                                       -command { ::spy::open_window } ]]]]
#        }

option add *Spy.timestampforeground Black         widgetDefault
option add *Spy.nickforeground      DarkBlue      widgetDefault
option add *Spy.jidforeground       DarkBlue      widgetDefault
option add *Spy.presenceforeground  DarkRed       widgetDefault
option add *Spy.reasonforeground    DarkMagenta   widgetDefault

namespace eval spy {
    set filename ""
    set the_file ""
    set watches {}
}

proc spy::open_window {} {
    global font
    variable watches

    set w .spy
    if {[winfo exists $w]} {
        return
    }

    add_win $w -title [::msgcat::mc "Presence Spy"] \
        -tabtitle [::msgcat::mc "Spy"] \
        -class Spy

    set sw [ScrolledWindow $w.isw -scrollbar vertical]
    pack $sw -side top -fill both -expand yes -in $w

    frame $w.controls
    button $w.controls.add -text [::msgcat::mc "Set watch"] \
      -command "[namespace current]::set_watch"

    set watch_entry [entry $w.controls.watch_regex]
    bind $watch_entry <Return> "[namespace current]::set_watch"
    pack $w.controls.add -padx 2m -pady 2m -side left -anchor e     
    pack $watch_entry -padx 2m -pady 2m -side left -anchor w -fill x
    pack $w.controls -side bottom -fill x

    frame $w.watches
    foreach watch $watches {
        set watch_id [lindex $watch 0]
        set watch_regex [lindex $watch 1]
        add_watch_frame $watch_id $watch_regex
    }
    pack $w.watches -side bottom -fill x
    
    set log [text $w.log -font $font]
    $sw setwidget $log

    $log configure -state disabled

    $log tag configure timestamp \
	-foreground [option get $w timestampforeground Spy]
    $log tag configure nick \
	-foreground [option get $w nickforeground Spy]
    $log tag configure jid \
	-foreground [option get $w jidforeground Spy]
    $log tag configure presence \
	-foreground [option get $w presenceforeground Spy]
    $log tag configure reason \
	-foreground [option get $w reasonforeground Spy]
}

proc spy::set_watch {} {
    variable watches
    
    set w .spy

    if {![winfo exists $w]} return
    set regex [$w.controls.watch_regex get]

    if { $regex == "" } return

    set next_watch [expr [llength $watches] + 1]
    lappend watches [list $next_watch $regex]
    add_watch_frame $next_watch $regex
}

proc spy::add_watch_frame {watch_id watch_regex} {
    variable spy_alerts
    set w .spy

    if {![winfo exists $w]} return

    frame $w.watches.$watch_id
    button $w.watches.$watch_id.remove -text [::msgcat::mc "Remove"] \
      -command [list [namespace current]::remove_watch $watch_id $watch_regex]
    label $w.watches.$watch_id.regex -text $watch_regex \
      -foreground [option get $w nickforeground Spy]
    label $w.watches.$watch_id.nick -text "" \
      -foreground [option get $w nickforeground Spy]
    label $w.watches.$watch_id.jid -text "" \
      -foreground [option get $w nickforeground Spy]
    label $w.watches.$watch_id.timestamp -text "" \
      -foreground [option get $w timestampforeground Spy]
    label $w.watches.$watch_id.presence -text "" \
      -foreground [option get $w presenceforeground Spy]
    label $w.watches.$watch_id.reason -text "" \
	  -foreground [option get $w reasonforeground Spy]
    checkbutton $w.watches.$watch_id.alert -text [::msgcat::mc "Alert when online?"] \
      -variable [namespace current]::spy_alerts($watch_id)
    
    pack $w.watches.$watch_id.remove \
      $w.watches.$watch_id.regex \
      $w.watches.$watch_id.timestamp \
      $w.watches.$watch_id.nick \
      $w.watches.$watch_id.jid \
      $w.watches.$watch_id.presence \
      $w.watches.$watch_id.reason -side left -fill x
    pack $w.watches.$watch_id.alert -side right
    pack $w.watches.$watch_id -side bottom -fill x

}

proc spy::remove_watch {watch_id watch_regex} {
    variable watches

    set w .spy.watches.$watch_id
    if {![winfo exists $w]} return

    set idx [lsearch -exact $watches [list $watch_id $watch_regex]]
    set watches [lreplace $watches $idx $idx]
    destroy $w
}

proc spy::update_watch {connid watch_id nick jid type reason} {
    variable spy_alerts
    set w .spy.watches.$watch_id
    if {![winfo exists $w]} return

    $w.timestamp configure -text [clock format [clock seconds]]
    $w.nick configure -text $nick
    $w.jid configure -text $jid
    $w.presence configure -text $type
    $w.reason configure -text $reason

    if { $spy_alerts($watch_id) && $type=="online" } {
        alert_dialog $connid $watch_id $nick $jid
    }
}

proc spy::alert_dialog {connid watch_id nick jid} {
    set w .spy_alert_${watch_id}

    if {[winfo exists $w]} {
        destroy $w
    }

    set message [format [::msgcat::mc "Spy Alert: user %s (%s) in online!"] $nick $jid]
    Dialog $w -title $message -separator 1 -anchor e -default 0

    set frame [$w getframe]
    message $frame.msg -text $message -width 70c
    pack $frame.msg -side left -fill x

    $w add -text [::msgcat::mc "Open chat"] -command [list [namespace current]::open_chat $w $connid $jid ]
    $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]

    $w draw
}

proc spy::open_chat {w connid jid} {
    destroy $w

    chat::open_to_user $connid $jid
}


proc spy::display {connid jid nick type reason} {
    variable watches
    set w .spy

    if {![winfo exists $w]} return
    set log $w.log

    $log configure -state normal

    $log insert end "[clock format [clock seconds]]" timestamp
    $log insert end " "

    if { "$nick"!="" } {
        $log insert end "$nick" nick
        $log insert end " ("
        $log insert end "$jid" jid
        $log insert end ") "
    } else {
        $log insert end "$jid" jid
        $log insert end " "
    }

    $log insert end "$type" presence

    if { "$reason"!="" } {
        $log insert end " ("
        $log insert end "$reason" reason
        $log insert end ")"
    }
    $log insert end "\n"
    $log see end

    $log configure -state disabled

    foreach watch $watches {
        set watch_id [lindex $watch 0]
        set watch_regex [lindex $watch 1]
        if {[regexp "$watch_regex" $nick temp] || [regexp "$watch_regex" $jid temp]} {
            update_watch $connid $watch_id $nick $jid $type $reason
        }
    }
}

proc spy::client_presence_handler {connid from type x opts} {
    variable the_file

    if {[catch { chat::get_nick $connid $from chat } nick]} {
	if {[catch { chat::get_nick $from chat } nick]} {
	    set nick $from
	}
    }

    if { "$nick"=="$from" } {
        set nick ""
    }

    set reason ""
    set show ""
    foreach {attr val} $opts {
	switch -- $attr {
	    -status   {set reason $val}
	    -show     {set show   $val}
	}
    }

    if { "$type"=="" } { set type "online" }
    if { "$show"!="" } { set type "$type/$show" }

    display "$connid" "$from" "$nick" "$type" "$reason"

    if { "$the_file"!="" } {
        puts -nonewline $the_file "[clock format [clock seconds]] "
        if { "$nick"!="" } {
            puts -nonewline $the_file "$nick ($from) "
        } else {
            puts -nonewline $the_file "$from "
        }
        puts -nonewline $the_file "$type"
        if { "$reason"!="" } {
            puts -nonewline $the_file " ($reason)"
        }
        puts $the_file {}
        flush $the_file
    }
}

proc spy::init_spy {} {
    variable the_file
    variable filename

    if { "$filename"!="" } {
        if { "$the_file"!="" } {
            close $the_file
            set the_file ""
        }
        set the_file [open "$filename" "a"]
        puts $the_file "[clock format [clock seconds]] Started spying."
    }
}

proc spy::deinit_spy {} {
    variable the_file
    if { "$the_file"!="" } {
        puts $the_file "[clock format [clock seconds]] Spy goes away."
        close $the_file
    }
}

proc spy::setup_menu {} {
    catch { 
        set m [.mainframe getmenu roster]

        $m add command -label [::msgcat::mc "Spy presence"] \
	    -command [namespace current]::open_window
    }
}

hook::add postload_hook [namespace current]::spy::init_spy 100
hook::add client_presence_hook [namespace current]::spy::client_presence_handler 100
hook::add quit_hook [namespace current]::spy::deinit_spy 100
hook::add finload_hook [namespace current]::spy::setup_menu
